#include "script_core.hpp"

#include "pw/core/vector.hpp"
#include "pw/core/quaternion.hpp"
#include "pw/core/axisangle.hpp"
#include "pw/core/debug.hpp"
#include "pw/core/size.hpp"
#include "pw/core/point.hpp"
#include "pw/core/timer.hpp"
#include "pw/core/mesh.hpp"

namespace pw {

void script_core::load(sol::table &ns)
{

    typedef real_t Scalar;

    ns.set("pi",pw::pi<Scalar>());


	ns.new_usertype<vector3>
			(
				"vector3",
				sol::constructors<vector3(),vector3(vector3::value_type,vector3::value_type,vector3::value_type)>(),
				"x", sol::property(sol::resolve<const vector3::value_type&() const>(&vector3::x), [](vector3& v,vector3::value_type val){ v.x() = val;}),
				"y", sol::property(sol::resolve<const vector3::value_type&() const>(&vector3::y), [](vector3& v,vector3::value_type val){ v.y() = val;}),
				"z", sol::property(sol::resolve<const vector3::value_type&() const>(&vector3::z), [](vector3& v,vector3::value_type val){ v.z() = val;}),
				"cross",&vector3::cross,
				"lerp",&vector3::lerp
			);



	ns.new_usertype<quaternion>
			(
				"quaternion",
				sol::constructors<quaternion(), quaternion(quaternion::value_type,quaternion::value_type,quaternion::value_type,quaternion::value_type)>(),
				"x", sol::property(sol::resolve<const quaternion::value_type&() const>(&quaternion::x), [](quaternion& v,quaternion::value_type val){ v.x() = val;}),
				"y", sol::property(sol::resolve<const quaternion::value_type&() const>(&quaternion::y), [](quaternion& v,quaternion::value_type val){ v.y() = val;}),
				"z", sol::property(sol::resolve<const quaternion::value_type&() const>(&quaternion::z), [](quaternion& v,quaternion::value_type val){ v.z() = val;}),
				"w", sol::property(sol::resolve<const quaternion::value_type&() const>(&quaternion::w), [](quaternion& v,quaternion::value_type val){ v.w() = val;}),
				"identity",sol::readonly_property(&quaternion::identity),
				"dot",&quaternion::dot,
				"inverse",scripting::readonly_property(&quaternion::inverse),
				"normalized",&quaternion::normalized,
				"lerp",&quaternion::lerp,
				"slerp",&quaternion::slerp
			);

	ns.new_usertype<axisangle>
		("axisangle",
			sol::constructors<axisangle(), axisangle(vector3,Scalar)>(),
			"axis",scripting::property(&axisangle::axis,&axisangle::set_axis),
			"angle",scripting::property(&axisangle::angle,&axisangle::set_angle)
		);


    ns.new_usertype<size>("size",
                          sol::constructors<size(),size(Scalar,Scalar)>(),
                          "width",&size::width,
                          "height",&size::height
                          //						   "none",sol::debug::level::none
                          );

    ns.new_usertype<point>("point",
                           sol::constructors<point(),point(Scalar,Scalar)>(),
                           "x",&point::x,
                           "y",&point::y
                           );


    ns.new_usertype<debug>("debug",
                           "new",sol::no_constructor,
                           "get",&debug::get,
                           "write",&debug::write
                           //						   "none",sol::debug::level::none
                           );


    ns.new_usertype<timer>("timer",
                           "now",sol::readonly_property(&timer::now),
                           "elapsed",sol::readonly_property(&timer::elapsed),
                           "reset",&timer::reset
                           );

	ns.new_usertype<mesh>
			(
				"mesh",
				sol::constructors<mesh()>()
				);

//	ns.new_enum<mesh::topology_type>
//			(
//				"mesh",
//				sol::constructors<mesh()>()
//				);
}





}
